/******************************************************************************
 *
 * Copyright:
 *    (C) 2005 Embedded Artists AB
 *
 * File:
 *    main.c
 *
 * Description:
 *    Sample application that demonstrates how to use a general
 *    interrupt handler that execute all interrupts in SYSTEM mode
 *    instead of IRQ mode.
 *
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/
#include "general.h"
#include <lpc2xxx.h>
#include <printf_P.h>
#include <ea_init.h>
#include <framework.h>
#include "startup/config.h"

/******************************************************************************
 * Defines and typedefs
 *****************************************************************************/
#define CRYSTAL_FREQUENCY FOSC
#define PLL_FACTOR        PLL_MUL
#define VPBDIV_FACTOR     PBSD

#define SYSTEM_TIME_TICK  1    //1Hz tick frequency


/*****************************************************************************
 * Public function prototypes
 ****************************************************************************/
int main(void);


/*****************************************************************************
 * External function prototypes
 ****************************************************************************/
void _generalIrqHandler(void);


/*****************************************************************************
 * Local function prototypes
 ****************************************************************************/
static void delayMs(tU32 delayInMs);
static void timer0ISR(void);


/*****************************************************************************
 * Implementation of local functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    Actual timer ISR that is called whenever timer 0 generated an interrupt.
 *
 *    NOTE: This is an ordinary C function and shall not be declared as an
 *          interrupt function!
 *
 ****************************************************************************/
static void
timer0ISR(void)
{
  tU32 executionMode;

  asm volatile ("mrs %0, cpsr"
                : "=r" (executionMode)
                :
               );

  //mask of mode bits
  executionMode &= 0x0000001f;
  
	printf("\nTimer interrupt that execute in mode: %x", executionMode);
	printf("\n(0x10=USER, 0x11=FIQ, 0x12=IRQ, 0x13=SVC, 0x17=ABT, 0x1B=UND, 0x1f=SYSTEM)\n");

  //end interrupt
  T0IR        = 0xff;        //reset all IRQ flags
  VICVectAddr = 0x00;        //dummy write to VIC to signal end of interrupt
}


/*****************************************************************************
 *
 * Description:
 *    Delay execution by a specified number of milliseconds by using
 *    timer #1. A polled implementation.
 *
 * Params:
 *    [in] delayInMs - the number of milliseconds to delay.
 *
 ****************************************************************************/
static void
delayMs(tU32 delayInMs)
{
  /*
   * setup timer #1 for delay
   */
  TIMER1_TCR = 0x02;          //stop and reset timer
  TIMER1_PR  = 0x00;          //set prescaler to zero
  TIMER1_MR0 = delayInMs * ((CRYSTAL_FREQUENCY * PLL_FACTOR)/ (1000 * VPBDIV_FACTOR));
  TIMER1_IR  = 0xff;          //reset all interrrupt flags
  TIMER1_MCR = 0x04;          //stop timer on match
  TIMER1_TCR = 0x01;          //start timer
  
  //wait until delay time has elapsed
  while (TIMER1_TCR & 0x01)
    ;
}


/*****************************************************************************
 * Implementation of public functions
 ****************************************************************************/

/*****************************************************************************
 *
 * Description:
 *    The routine is called whenever an IRQ occurs, and it should dispatch the 
 *    correct IRQ handler (with information from VIC). 
 *
 ****************************************************************************/
void
handleIRQs(void)
{
  void (*pfnct)(void);                   /* pfnct is declared to be a function pointer      */
  
  pfnct = (void (*)(void))VICVectAddr;   /* read interrupt vector from the VIC              */

  /* loop through all currently active interrupts */
  while (pfnct != (void (*)(void))0)
  {
    (*pfnct)();                          /* call the registered ISR for interrupting device */
    pfnct = (void (*)(void))VICVectAddr; /* read next interrupt vector from the VIC         */
  }
}


/*****************************************************************************
 *
 * Description:
 *    The main-function. 
 *
 * Returns:
 *    Always 0, since return value is not used.
 *
 ****************************************************************************/
int
main(void)
{
  //initialize printf()-functionality
  eaInit();

  //print welcome message
  printf("\n*********************************************************");
  printf("\n*");
  printf("\n* (C) 2005 Embedded Artists AB");
  printf("\n*");
  printf("\n* Welcome to this program that demonstrates how to");
  printf("\n* execute interrupts in system mode (instead of irq mode).");
  printf("\n*");
  printf("\n*********************************************************\n");

  //install a general interrupt
  pISR_IRQ = (tU32)_generalIrqHandler;
  
  //initialize interrupt controller
  VICIntSelect &= ~0x00000010;
  VICIntEnable |=  0x00000010;
  VICVectAddr4  = (tU32)timer0ISR;
  VICVectCntl4  =  0x00000024;

  //initialize and start Timer0 (1 Hz tick rate)
  T0TCR = 0x00000002;                           //disable and reset Timer0
  T0PC  = 0x00000000;                           //no prescale of clock
  T0MR0 = (CRYSTAL_FREQUENCY * PLL_FACTOR) /    //calculate no of timer ticks
          (SYSTEM_TIME_TICK * VPBDIV_FACTOR);   //for a given system tick rate
  T0IR  = 0x000000ff;                           //reset all flags before enable IRQs
  T0MCR = 0x00000003;                           //reset counter and generate IRQ on MR0 match
  T0TCR = 0x00000001;                           //start Timer0

  //enter forever loop...
  while(1)
  {
  	printf(".");

    //wait 200 ms
    delayMs(200);
  }

  return 0;
}
